/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

GtkWidget* create_selectkb_window (void);
GtkWidget* create_main_window (void);
GtkWidget* create_get_scancode_dialog (void);
GtkWidget* create_new_key_name_dialog (void);
GtkWidget* create_contact_dialog (void);
GtkWidget* create_about_dialog (void);
